! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program checkorbit

  ! 
  ! short f90 program to test Orbit 
  ! Author: BHP, Leicester bhp@star.le.ac.uk
  ! 14 June 1999 -- first release
  ! 14 Feb  2000 -- third release
  ! 14 Feb  2014 -- fourth release (wow, 15 years and counting!)

  use dal
  use oal
  use errorhandling
  use saslib
  
  implicit none
  
  type(DataSetT)  :: in_set
  type(TableT)    :: in_tab
  type(ColumnT)   :: time, x, y, z, vx , vy , vz
  type(OdfInfoType) :: odfinfo
  real(double), dimension(:), pointer :: t_ptr, x_ptr , y_ptr , z_ptr , &
                                         vx_ptr , vy_ptr , vz_ptr
  real(kind=double) :: obsstart , obsend
  integer nrows , i
  write (*,*) "testing orbit.fit file..."
  
  in_set = dataSet("orbit.fit", read)
  in_tab = table(in_set,"ORBIT")
  nrows = numberofRows(in_tab)
  
  if (.not. hasColumn(in_tab,"TIME") .OR. &
       .not. hasColumn(in_tab,"GEI_X") .OR. &
       .not. hasColumn(in_tab,"GEI_Y") .OR. &
       .not. hasColumn(in_tab,"GEI_Z") .OR. &
       .not. hasColumn(in_tab,"GSE_X") .OR. &
       .not. hasColumn(in_tab,"GSE_Y") .OR. &
       .not. hasColumn(in_tab,"GSE_Z") .OR. &
       .not. hasColumn(in_tab,"VX") .OR. &
       .not. hasColumn(in_tab,"VY") .OR. &
       .not. hasColumn(in_tab,"VZ")) then

     call error("noColumn","TIME, POSITION or VELOCITY column(s) is missing")
  end if
  
  time = column(in_tab,"TIME",read)

!  x = column(in_tab,"X",read)
!  y = column(in_tab,"Y",read)
!  z = column(in_tab,"Z",read)
!  vx = column(in_tab,"VX",read)
!  vy = column(in_tab,"VY",read)
!  vz = column(in_tab,"VZ",read)
  t_ptr => real64Data(time)
!  x_ptr => real64Data(x)
!  y_ptr => real64Data(y)
!  z_ptr => real64Data(z)
!  vx_ptr => real64Data(vx)
!  vy_ptr => real64Data(vy)
!  vz_ptr => real64Data(vz)

  call OAL_odfInfo(odfinfo)
  obsstart = toTimeTag(odfinfo%observationStartTime)
  obsend = toTimeTag(odfinfo%observationEndTime)

  !  do i = 0 , nrows-1  
  !     write(errstr,'(3(f12.3,1x))') obsstart, t_ptr(i), obsend
  !     call message(AppMsg,Verbose,trim(errstr))
  !  end do

  if (.not. (t_ptr(0) >= obsstart) .OR. .not. (t_ptr(nrows-1) <= obsend)) then
     write(errstr,'(5(f12.3,1x))') obsstart, t_ptr(0), obsend, t_ptr(nrows-1), t_ptr(nrows-2)
     call error("BadTime(s)",trim(errstr))
  else 
     write(errstr,*) "CHECKORBIT: times ok within boundaries"
     call message(AppMsg,Verbose)  
  end if
  call release(time)
  call release(in_tab)
  call release(in_set)
  
!  if (.not. (x_ptr(0) == HUGE(1.0d0)) .OR. &
!       .not. (y_ptr(0) == HUGE(1.0d0)) .OR. &
!       .not. (z_ptr(0) == HUGE(1.0d0)) .OR. &
!       .not. (vx_ptr(0) == HUGE(1.0d0)) .OR. &
!       .not. (vy_ptr(0) == HUGE(1.0d0)) .OR. &
!       .not. (vz_ptr(0) == HUGE(1.0d0))) then
!     call error("BadPOSVEL","Position or Velocity values not as input")
!  end if
  

!  call release(x)
!  call release(y)
!  call release(z)
!  call release(vx)
!  call release(vy)
!  call release(vz)
  
end program checkorbit

